/* SDCal, version 01
 * Copyright (C) 2022 Arktis Desktop Project, GNU GPL v2 or any later.*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <locale.h>
#include "SDCal.h"
#include "Defaults.h"

options option;
static char program_path[STRING_XL];
static void Version(void);
static void SetOptions(int, char *[]);
static void StringCopyXL(char *, const char *);
static void StringCopyM(char *, const char *);
static void StringCopyS(char *, const char *);

int main(int argc, char *argv[]) {
	setlocale(LC_ALL, "");
	StringCopyXL(program_path, argv[0]);
	SetOptions(argc, argv);
	CreateWindow(argc, argv);
	PaintCal();
	if (option.pix)
		ShowCal();
	return EXIT_SUCCESS;
}

static void Version(void) {
	printf("SDCal, version 01\n");
	printf("Copyright (C) 2022 Arktis Desktop Project, ");
	printf("GNU GPL v2 or any later.\n");
	printf("Please see /root/.SDCalendar/data/README for details. \n");
}

static void SetOptions(int ac, char *av[]) {
#define OPT_DSHAD 0x101
#define OPT_HFONT 0x102
#define OPT_UFONT 0x103
#define OPT_HSHAD 0x104
#define OPT_ASHAD 0x105
#define OPT_ACOLR 0x106
#define OPT_ICOLR 0x107
#define OPT_FFONT 0x108
#define OPT_DAYSW 0x109
#define OPT_LCNTR 0x110
#define OPT_MSCOL 0x111
	int opt_index = 0, o;
	static char *short_opts = "A:a:B:b:C:c:D:d:E:e:F:f:G:g:hI:i:J:j:K:k:L:l:M:m:Nn:O:o:P:p:q:R:r:S:s:T:t:U:u:vW:w:";
	static struct option long_opts[] =
	{
	{"aa", 0, 0, OPT_ASHAD},
	{"ab", 0, 0, OPT_ACOLR},
	{"ac", 1, 0, 'A'},
	{"as", 1, 0, 'a'},
	{"ar", 1, 0, 'o'},
	{"as", 1, 0, 'a'},
	{"bg", 1, 0, 'B'},
	{"ca", 1, 0, 'J'},
	{"cb", 1, 0, 'j'},
	{"cd", 0, 0, OPT_MSCOL},
	{"cl", 1, 0, 'n'},
	{"dd", 0, 0, OPT_UFONT},
	{"dl", 1, 0, 'r'},
	{"dx", 1, 0, 'P'},
	{"dy", 1, 0, 'p'},
	{"ee", 1, 0, 'e'},
	{"ei", 0, 0, OPT_DAYSW},
	{"fd", 1, 0, 'R'},
	{"fn", 1, 0, 'F'},
	{"fh", 1, 0, 'E'},
	{"ff", 1, 0, 'f'},
	{"fy", 0, 0, OPT_FFONT},
	{"hc", 1, 0, 'c'},
	{"help", 0, 0, 'h'},
	{"hh", 0, 0, OPT_HFONT},
	{"hl", 1, 0, 'L'},
	{"hs", 1, 0, 's'},
	{"hx", 1, 0, 'G'},
	{"hy", 1, 0, 'g'},
	{"ic", 1, 0, 'I'},
	{"is", 1, 0, 'i'},
	{"ii", 0, 0, OPT_ICOLR},
	{"ma", 1, 0, 'O'},
	{"mb", 1, 0, 'b'},
	{"mc", 1, 0, 'M'},
	{"mp", 1, 0, 'q'},
	{"mw", 1, 0, 'm'},
	{"pix", 0, 0, 'N'},
	{"pm", 1, 0, 'T'},
	{"py", 1, 0, 't'},
	{"rh", 1, 0, 'l'},
	{"rx", 1, 0, 'K'},
	{"ry", 1, 0, 'k'},
	{"sc", 1, 0, 'S'},
	{"sh", 0, 0, OPT_HSHAD},
	{"ss", 0, 0, OPT_DSHAD},
	{"tc", 1, 0, 'C'},
	{"ua", 1, 0, 'U'},
	{"ub", 1, 0, 'u'},
	{"version", 0, 0, 'v'},
	{"wa", 1, 0, 'W'},
	{"wb", 1, 0, 'w'},
	{"xc", 1, 0, 'D'},
	{"yc", 1, 0, 'd'},
	{NULL, 0, 0, 0}
	};
/* Begin by setting the default options (defined in Defaults.h). */
	option.bg_pix[0] = '\0';
	option.day_pix[0] = '\0';
	StringCopyS(option.ac, DATEC);
	StringCopyS(option.as, DATES);
	StringCopyS(option.ca, DATEC);
	StringCopyS(option.cb, DATES);
	StringCopyM(option.fd, DFONT);
	StringCopyM(option.ff, DFONT);
	StringCopyM(option.fh, DFONT);
	StringCopyM(option.fn, DFONT);
	StringCopyS(option.hc, DATEC);
	StringCopyM(option.hl, MOLBL);
	StringCopyS(option.hs, DATES);
	StringCopyS(option.ic, DATEC);
	StringCopyS(option.is, DATES);
	StringCopyS(option.mc, LINEC);
	StringCopyS(option.mb, SATCS);
	StringCopyM(option.rh, YRLBL);
	StringCopyS(option.sc, DATES);
	StringCopyS(option.tc, DATEC);
	StringCopyS(option.ua, SATCT);
	StringCopyS(option.ub, SATCS);
	StringCopyS(option.wa, SUNCT);
	StringCopyS(option.wb, SUNCS);
	option.aa = ASHAD;
	option.ab = ACOLR;
	option.ar = ARCSW;
	option.cd = MSCOL;
	option.cl = LCNTR;
	option.dl = DLSEP;
	option.dx = DXSEP;
	option.dy = DYSEP;
	option.dd = UFONT;
	option.ei = DAYSW;
	option.fy = FFONT;
	option.hh = HFONT;
	option.hx = MNTHX;
	option.hy = HEADY;
	option.ii = ICOLR;
	option.mp = DATEP;
	option.mw = LINEW;
	option.pix = PRINT;
	option.pm = CMNTH;
	option.py = CYEAR;
	option.rx = YEARX;
	option.ry = HEADY;
	option.sh = HSHAD;
	option.ss = DSHAD;
	option.xc = XSCAL;
	option.yc = YSCAL;
/* Loop through the user-provided options.*/
	while ((o = getopt_long(ac, av, short_opts, long_opts, &opt_index)) != EOF) {
		switch (o) {
			case 'A':
				StringCopyS(option.ac, optarg);
				option.ab = !ACOLR;
				break;
			case 'a':
				StringCopyS(option.as, optarg);
				option.aa = !ASHAD;
				break;
			case 'B':
				StringCopyM(option.bg_pix, optarg);
				break;
			case 'b':
				StringCopyS(option.mb, optarg);
				break;
			case 'C':
				StringCopyS(option.tc, optarg);
				break;
			case 'c':
				StringCopyS(option.hc, optarg);
				break;
			case 'D':
				option.xc = atoi(optarg);
				break;
			case 'd':
				option.yc = atoi(optarg);
				break;
			case 'F':
				StringCopyM(option.fn, optarg);
				break;
			case 'f':
				option.fy = !FFONT;
				StringCopyM(option.ff, optarg);
				break;
			case 'E':
				option.hh = !HFONT;
				StringCopyM(option.fh, optarg);
				break;
			case 'e':
				option.ei = !DAYSW;
				option.ee = atoi(optarg);
				break;
			case 'G':
				option.hx = atoi(optarg);
				break;
			case 'g':
				option.hy = atoi(optarg);
				break;
			case 'h':
				Version();
				exit(EXIT_SUCCESS);
				break;
			case 'I':
				StringCopyS(option.ic, optarg);
				option.ii = !ICOLR;
				break;
			case 'i':
				StringCopyS(option.is, optarg);
				break;
			case 'J':
				StringCopyS(option.ca, optarg);
				break;
			case 'j':
				StringCopyS(option.cb, optarg);
				option.cd = !MSCOL;
				break;
			case 'L':
				StringCopyM(option.hl, optarg);
				break;
			case 'l':
				StringCopyM(option.rh, optarg);
				break;
			case 'K':
				option.rx = atoi(optarg);
				break;
			case 'k':
				option.ry = atoi(optarg);
				break;
			case 'M':
				StringCopyS(option.mc, optarg);
				break;
			case 'm':
				option.mw = atoi(optarg);
				break;
			case 'N':
				option.pix = !PRINT;
				break;
			case 'n':
				option.cl = atoi(optarg);
				break;
			case 'o':
				option.ar = atoi(optarg);
				break;
			case 'O':
				StringCopyM(option.day_pix, optarg);
				option.ar = 6;
				break;
			case 'P':
				option.dx = atoi(optarg);
				break;
			case 'p':
				option.dy = atoi(optarg);
				break;
			case 'q':
				option.mp = atoi(optarg);
				break;
			case 'R':
				option.dd = !UFONT;
				StringCopyM(option.fd, optarg);
				break;
			case 'r':
				option.dl = atoi(optarg);
				break;
			case 'S':
				StringCopyS(option.sc, optarg);
				option.ss = !DSHAD;
				break;
			case 's':
				StringCopyS(option.hs, optarg);
				option.sh = !HSHAD;
				break;
			case 'T':
				option.pm = atoi(optarg);
				break;
			case 't':
				option.py = atoi(optarg);
				break;
			case 'v':
				Version();
				exit(EXIT_SUCCESS);
				break;
			case 'U':
				StringCopyS(option.ua, optarg);
				break;
			case 'u':
				StringCopyS(option.ub, optarg);
				option.aa = !ASHAD;
				break;
			case 'W':
				StringCopyS(option.wa, optarg);
				break;
			case 'w':
				StringCopyS(option.wb, optarg);
				option.aa = !ASHAD;
				break;
			default:
				exit(EXIT_FAILURE);
		}
	}
	if (optind < ac)
	fprintf(stderr, "SDCal: unknown argument \n");
}

static void StringCopyXL(char *destination, const char *source) {
	strncpy(destination, source, STRING_XL);
	destination[STRING_XL - 1] = '\0';
}

static void StringCopyM(char *destination, const char *source) {
	strncpy(destination, source, STRING_M);
	destination[STRING_M - 1] = '\0';
}

static void StringCopyS(char *destination, const char *source) {
	strncpy(destination, source, STRING_S);
	destination[STRING_S - 1] = '\0';
}

