/* SDCalX, version 01
 * Copyright (C) 2022 Arktis Desktop Project, GNU GPL v2 or any later.*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>
#include "SDCalX.h"

static Display *display;
static Window root, window;
static Atom aa, ab, ac, ad, ae[4];
static Pixmap bg_pm, mask_pm;

void CreateWindow(int xx, int yy, int ww, int hh) {
	int zero = 0;
	char *app_name = NAME;
	char *app_class = CLASS;
	int screen;
	XSizeHints size_hints;
	XClassHint class_hints;
	size_hints.flags = PMinSize | PMaxSize | PPosition;
	size_hints.min_width = size_hints.max_width = ww;
	size_hints.min_height = size_hints.max_height = hh;
	display = XOpenDisplay(NULL);
	if (display == NULL) {
		fprintf(stderr, "SDCalX: could not open display ``%s''..Exit.\n",
				XDisplayName(NULL));
		exit(EXIT_FAILURE);
	}
	screen = DefaultScreen(display);
	root = RootWindow(display, screen);
	bg_pm = XCreatePixmap(display, root, ww, hh,
							DefaultDepth(display, screen));
	XWMGeometry(display, screen, "", NULL, 0, &size_hints, &zero, &zero,
				&size_hints.width, &size_hints.height, &zero);
	window = XCreateSimpleWindow(display, root, xx, yy, 
			size_hints.width, size_hints.height, 0, 
			BlackPixel(display, screen), BlackPixel(display, screen));
	XSetWMNormalHints(display, window, &size_hints);
	class_hints.res_name = app_name;
	class_hints.res_class = app_class;
	XSetClassHint(display, window, &class_hints);
	aa = XInternAtom(display, "WM_DELETE_WINDOW", FALSE);
	XSetWMProtocols(display, window, &aa, 1);
	ab = XInternAtom(display, "_NET_WM_WINDOW_TYPE", FALSE);
	ac = XInternAtom(display, "_NET_WM_WINDOW_TYPE_SPLASH", FALSE);
	XChangeProperty(display, window, ab, XA_ATOM, 32, PropModeReplace,
					(unsigned char *) &ac, 1);
	ad = XInternAtom(display, "_NET_WM_STATE", False);
	ae[0] = XInternAtom(display, "_NET_WM_STATE_SKIP_PAGER", False);
	ae[1] = XInternAtom(display, "_NET_WM_STATE_SKIP_TASKBAR", False);
	ae[2] = XInternAtom(display, "_NET_WM_STATE_STICKY", False);
	ae[3] = XInternAtom(display, "_NET_WM_STATE_BELOW", False);
	XChangeProperty(display, window, ad, XA_ATOM, 32, PropModeAppend, 
					(unsigned char *) &ae, 4);
	XSelectInput(display, window, StructureNotifyMask);
	XStoreName(display, window, app_name);
	XSetIconName(display, window, app_name);
	char pixdata[32];
	sprintf(pixdata, "/root/.SDCalendar/tmp/bg.xpm");
	int result;
	XpmAttributes attributes;
	attributes.valuemask = XpmSize;
	result = XpmReadFileToPixmap(display, root, pixdata, &bg_pm, &mask_pm, 
									&attributes);
	if (result != XpmSuccess) {
		fprintf(stderr, "SDCalX: trouble loading pixmap.. exit.\n");
		exit(EXIT_FAILURE);
	}
	XSetWindowBackgroundPixmap(display, window, bg_pm);
	XShapeCombineMask(display, window, ShapeBounding, 0, 0, mask_pm, 
						ShapeSet);
	XMapWindow(display, window);
}

void 
RePaint(int *mdnite) {
	*mdnite = FALSE;
	struct tm *time_struct;
	time_t curtime;
	curtime = time(NULL);
	time_struct = localtime(&curtime);
	int h = time_struct->tm_hour;
	int m = time_struct->tm_min;
	if (h == 0 && m == 0) 
		*mdnite = TRUE;
}

void
HandleEvents(int *should_quit) {
	XEvent event;
	*should_quit = FALSE;
	while (XPending(display)) {
		XNextEvent(display, &event);
		switch (event.type) {
			case DestroyNotify:
				*should_quit = TRUE;
				break;
			case ClientMessage:
				if (event.xclient.data.l[0] == aa)
					*should_quit = TRUE;
				break;
			default:
				break;
		}
	}
}

void
DestroyWindow(void) {
	XFreePixmap(display, bg_pm);
	XDestroyWindow(display, window);
	XCloseDisplay(display);
}

